@@ -31,6 +31,7 @@
 #include <string.h>
 #include <sys/ioctl.h>
 #include <arpa/inet.h>
+#include <sys/time.h>
 
 #include <netpacket/packet.h>
 #include <netinet/if_ether.h>
@@ -72,6 +73,7 @@ typedef enum _dhcp_client_state {
 	INFORMATION_REQ,
 	SOLICITATION,
 	REQUEST,
+	CONFIRM,
 	RENEW,
 	REBIND,
 	RELEASE,
@@ -130,6 +132,8 @@ struct _GDHCPClient {
 	gpointer rebind_data;
 	GDHCPClientEventFunc release_cb;
 	gpointer release_data;
+	GDHCPClientEventFunc confirm_cb;
+	gpointer confirm_data;
 	char *last_address;
 	unsigned char *duid;
 	int duid_len;
@@ -143,6 +147,8 @@ struct _GDHCPClient {
 	time_t last_renew;
 	time_t last_rebind;
 	time_t expire;
+	gboolean retransmit;
+	struct timeval start_time;
 };
 
 static inline void debug(GDHCPClient *client, const char *format, ...)
@@ -249,13 +255,27 @@ static void copy_option(uint8_t *buf, uint16_t code, uint16_t len,
 		memcpy(&buf[4], msg, len);
 }
 
+static int32_t get_time_diff(struct timeval *tv)
+{
+	struct timeval now;
+	int32_t hsec;
+
+	gettimeofday(&now, NULL);
+
+	hsec = (now.tv_sec - tv->tv_sec) * 100;
+	hsec += (now.tv_usec - tv->tv_usec) / 10000;
+
+	return hsec;
+}
+
 static void add_dhcpv6_request_options(GDHCPClient *dhcp_client,
 				struct dhcpv6_packet *packet,
 				unsigned char *buf, int max_buf,
 				unsigned char **ptr_buf)
 {
 	GList *list;
-	uint16_t code;
+	uint16_t code, value;
+	int32_t diff;
 	int len;
 
 	if (dhcp_client->type == G_DHCP_IPV4)
@@ -313,6 +333,31 @@ static void add_dhcpv6_request_options(GDHCPClient *dhcp_client,
 		case G_DHCPV6_ORO:
 			break;
 
+		case G_DHCPV6_ELAPSED_TIME:
+			if (dhcp_client->retransmit == FALSE) {
+				/*
+				 * Initial message, elapsed time is 0.
+				 */
+				diff = 0;
+			} else {
+				diff = get_time_diff(&dhcp_client->start_time);
+				if (diff < 0 || diff > 0xffff)
+					diff = 0xffff;
+			}
+
+			len = 2 + 2 + 2;
+			if ((*ptr_buf + len) > (buf + max_buf)) {
+				debug(dhcp_client, "Too long dhcpv6 message "
+					"when writing elapsed time option");
+				return;
+			}
+
+			value = htons((uint16_t)diff);
+			copy_option(*ptr_buf, G_DHCPV6_ELAPSED_TIME,
+				2, (uint8_t *)&value);
+			(*ptr_buf) += len;
+			break;
+
 		case G_DHCPV6_DNS_SERVERS:
 			break;
 
@@ -568,6 +613,22 @@ done:
 	close(sk);
 }
 
+void g_dhcpv6_client_set_retransmit(GDHCPClient *dhcp_client)
+{
+	if (dhcp_client == NULL)
+		return;
+
+	dhcp_client->retransmit = TRUE;
+}
+
+void g_dhcpv6_client_clear_retransmit(GDHCPClient *dhcp_client)
+{
+	if (dhcp_client == NULL)
+		return;
+
+	dhcp_client->retransmit = FALSE;
+}
+
 int g_dhcpv6_create_duid(GDHCPDuidType duid_type, int index, int type,
 			unsigned char **duid, int *duid_len)
 {
@@ -701,7 +762,7 @@ static void put_iaid(GDHCPClient *dhcp_client, int index, uint8_t *buf)
 
 int g_dhcpv6_client_set_ia(GDHCPClient *dhcp_client, int index,
 			int code, uint32_t *T1, uint32_t *T2,
-			gboolean add_iaaddr)
+			gboolean add_iaaddr, const char *ia_na)
 {
 	if (code == G_DHCPV6_IA_TA) {
 		uint8_t ia_options[4];
@@ -713,13 +774,29 @@ int g_dhcpv6_client_set_ia(GDHCPClient *dhcp_client, int index,
 					ia_options, sizeof(ia_options));
 
 	} else if (code == G_DHCPV6_IA_NA) {
+		struct in6_addr addr;
 
 		g_dhcp_client_set_request(dhcp_client, G_DHCPV6_IA_NA);
 
-		if (add_iaaddr == TRUE) {
+		/*
+		 * If caller has specified the IPv6 address it wishes to
+		 * to use (ia_na != NULL and address is valid), then send
+		 * the address to server.
+		 * If caller did not specify the address (ia_na == NULL) and
+		 * if the current address is not set, then we should not send
+		 * the address sub-option.
+		 */
+		if (add_iaaddr == TRUE && ((ia_na == NULL &&
+			IN6_IS_ADDR_UNSPECIFIED(&dhcp_client->ia_na) == FALSE)
+			|| (ia_na != NULL &&
+				inet_pton(AF_INET6, ia_na, &addr) == 1))) {
 #define IAADDR_LEN (16+4+4)
 			uint8_t ia_options[4+4+4+2+2+IAADDR_LEN];
 
+			if (ia_na != NULL)
+				memcpy(&dhcp_client->ia_na, &addr,
+						sizeof(struct in6_addr));
+
 			put_iaid(dhcp_client, index, ia_options);
 
 			if (T1 != NULL) {
@@ -781,6 +858,8 @@ int g_dhcpv6_client_set_oro(GDHCPClient *dhcp_client, int args, ...)
 
 	g_dhcpv6_client_set_send(dhcp_client, G_DHCPV6_ORO, values, len);
 
+	g_free(values);
+
 	return 0;
 }
 
@@ -795,13 +874,23 @@ static int send_dhcpv6_msg(GDHCPClient *dhcp_client, int type, char *msg)
 	packet = (struct dhcpv6_packet *)&buf[0];
 	ptr = buf + sizeof(struct dhcpv6_packet);
 
-	debug(dhcp_client, "sending DHCPv6 %s message", msg);
-
 	init_packet(dhcp_client, packet, type);
 
-	dhcp_client->xid = packet->transaction_id[0] << 16 |
-			packet->transaction_id[1] << 8 |
-			packet->transaction_id[2];
+	if (dhcp_client->retransmit == FALSE) {
+		dhcp_client->xid = packet->transaction_id[0] << 16 |
+				packet->transaction_id[1] << 8 |
+				packet->transaction_id[2];
+		gettimeofday(&dhcp_client->start_time, NULL);
+	} else {
+		packet->transaction_id[0] = dhcp_client->xid >> 16;
+		packet->transaction_id[1] = dhcp_client->xid >> 8 ;
+		packet->transaction_id[2] = dhcp_client->xid;
+	}
+
+	g_dhcp_client_set_request(dhcp_client, G_DHCPV6_ELAPSED_TIME);
+
+	debug(dhcp_client, "sending DHCPv6 %s message xid 0x%04x", msg,
+							dhcp_client->xid);
 
 	max_buf = MAX_DHCPV6_PKT_SIZE - sizeof(struct dhcpv6_packet);
 
@@ -825,6 +914,11 @@ static int send_dhcpv6_request(GDHCPClient *dhcp_client)
 	return send_dhcpv6_msg(dhcp_client, DHCPV6_REQUEST, "request");
 }
 
+static int send_dhcpv6_confirm(GDHCPClient *dhcp_client)
+{
+	return send_dhcpv6_msg(dhcp_client, DHCPV6_CONFIRM, "confirm");
+}
+
 static int send_dhcpv6_renew(GDHCPClient *dhcp_client)
 {
 	return send_dhcpv6_msg(dhcp_client, DHCPV6_RENEW, "renew");
@@ -1508,7 +1602,7 @@ static char *malloc_option_value_string(uint8_t *option, GDHCPOptionType type)
 		return NULL;
 	upper_length = len_of_option_as_string[type] *
 			((unsigned)len / (unsigned)optlen);
-	dest = ret = malloc(upper_length + 1);
+	dest = ret = g_malloc(upper_length + 1);
 	if (ret == NULL)
 		return NULL;
 
@@ -1821,6 +1915,7 @@ static gboolean listener_event(GIOChannel *channel, GIOCondition condition,
 	uint8_t *message_type = NULL, *client_id = NULL, *option,
 		*server_id = NULL;
 	uint16_t option_len = 0, status = 0;
+	uint32_t xid = 0;
 	gpointer pkt;
 	unsigned char buf[MAX_DHCPV6_PKT_SIZE];
 	uint16_t pkt_len = 0;
@@ -1837,6 +1932,8 @@ static gboolean listener_event(GIOChannel *channel, GIOCondition condition,
 
 	pkt = &packet;
 
+	dhcp_client->status_code = 0;
+
 	if (dhcp_client->listen_mode == L2)
 		re = dhcp_recv_l2_packet(&packet,
 					dhcp_client->listener_sockfd);
@@ -1846,9 +1943,14 @@ static gboolean listener_event(GIOChannel *channel, GIOCondition condition,
 						dhcp_client->listener_sockfd);
 			pkt_len = re;
 			pkt = packet6;
-		} else
+			xid = packet6->transaction_id[0] << 16 |
+				packet6->transaction_id[1] << 8 |
+				packet6->transaction_id[2];
+		} else {
 			re = dhcp_recv_l3_packet(&packet,
 						dhcp_client->listener_sockfd);
+			xid = packet.xid;
+		}
 	} else if (dhcp_client->listen_mode == L_ARP) {
 		ipv4ll_recv_arp_packet(dhcp_client);
 		return TRUE;
@@ -1895,9 +1997,7 @@ static gboolean listener_event(GIOChannel *channel, GIOCondition condition,
 				}
 			}
 			dhcp_client->status_code = status;
-		} else
-			dhcp_client->status_code = 0;
-
+		}
 	} else {
 		message_type = dhcp_get_option(&packet, DHCP_MESSAGE_TYPE);
 		if (message_type == NULL)
@@ -1908,8 +2008,8 @@ static gboolean listener_event(GIOChannel *channel, GIOCondition condition,
 		/* No message type / client id option, ignore package */
 		return TRUE;
 
-	debug(dhcp_client, "received DHCP packet (current state %d)",
-							dhcp_client->state);
+	debug(dhcp_client, "received DHCP packet xid 0x%04x "
+			"(current state %d)", xid, dhcp_client->state);
 
 	switch (dhcp_client->state) {
 	case INIT_SELECTING:
@@ -2033,6 +2133,7 @@ static gboolean listener_event(GIOChannel *channel, GIOCondition condition,
 	case RENEW:
 	case REBIND:
 	case RELEASE:
+	case CONFIRM:
 		if (dhcp_client->type != G_DHCP_IPV6)
 			return TRUE;
 
@@ -2056,7 +2157,6 @@ static gboolean listener_event(GIOChannel *channel, GIOCondition condition,
 
 		switch_listening_mode(dhcp_client, L_NONE);
 
-		dhcp_client->status_code = 0;
 		get_dhcpv6_request(dhcp_client, packet6, pkt_len,
 						&dhcp_client->status_code);
 
@@ -2089,6 +2189,30 @@ static gboolean listener_event(GIOChannel *channel, GIOCondition condition,
 					dhcp_client->release_data);
 			return TRUE;
 		}
+		if (dhcp_client->confirm_cb != NULL) {
+			count = 0;
+			server_id = dhcpv6_get_option(packet6, pkt_len,
+						G_DHCPV6_SERVERID, &option_len,
+						&count);
+			if (server_id == NULL || count != 1 ||
+							option_len == 0) {
+				/* RFC 3315, 15.10 */
+				debug(dhcp_client,
+					"confirm server duid error, "
+					"discarding msg %p/%d/%d",
+					server_id, option_len, count);
+				return TRUE;
+			}
+			dhcp_client->server_duid = g_try_malloc(option_len);
+			if (dhcp_client->server_duid == NULL)
+				return TRUE;
+			memcpy(dhcp_client->server_duid, server_id, option_len);
+			dhcp_client->server_duid_len = option_len;
+
+			dhcp_client->confirm_cb(dhcp_client,
+						dhcp_client->confirm_data);
+			return TRUE;
+		}
 		break;
 	default:
 		break;
@@ -2213,6 +2337,16 @@ int g_dhcp_client_start(GDHCPClient *dhcp_client, const char *last_address)
 			}
 			send_dhcpv6_request(dhcp_client);
 
+		} else if (dhcp_client->confirm_cb) {
+			dhcp_client->state = CONFIRM;
+			re = switch_listening_mode(dhcp_client, L3);
+			if (re != 0) {
+				switch_listening_mode(dhcp_client, L_NONE);
+				dhcp_client->state = 0;
+				return re;
+			}
+			send_dhcpv6_confirm(dhcp_client);
+
 		} else if (dhcp_client->renew_cb) {
 			dhcp_client->state = RENEW;
 			re = switch_listening_mode(dhcp_client, L3);
@@ -2399,6 +2533,12 @@ void g_dhcp_client_register_event(GDHCPClient *dhcp_client,
 		dhcp_client->release_cb = func;
 		dhcp_client->release_data = data;
 		return;
+	case G_DHCP_CLIENT_EVENT_CONFIRM:
+		if (dhcp_client->type == G_DHCP_IPV4)
+			return;
+		dhcp_client->confirm_cb = func;
+		dhcp_client->confirm_data = data;
+		return;
 	}
 }
 
@@ -2437,6 +2577,7 @@ char *g_dhcp_client_get_netmask(GDHCPClient *dhcp_client)
 	case INFORMATION_REQ:
 	case SOLICITATION:
 	case REQUEST:
+	case CONFIRM:
 	case RENEW:
 	case REBIND:
 	case RELEASE:
