@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2012  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -35,8 +35,6 @@ extern "C" {
 #endif
 
 typedef int		connman_bool_t;
-typedef unsigned char	connman_uint8_t;
-typedef unsigned short	connman_uint16_t;
 
 #ifdef __cplusplus
 }
