@@ -510,6 +510,7 @@ static void scan_callback(int result, GSupplicantInterface *interface,
 {
 	struct connman_device *device = user_data;
 	struct wifi_data *wifi = connman_device_get_data(device);
+	connman_bool_t scanning;
 
 	DBG("result %d wifi %p", result, wifi);
 
@@ -522,12 +523,23 @@ static void scan_callback(int result, GSupplicantInterface *interface,
 	if (result < 0)
 		connman_device_reset_scanning(device);
 
-	connman_device_set_scanning(device, FALSE);
+	scanning = connman_device_get_scanning(device);
+
+	if (scanning == TRUE)
+		connman_device_set_scanning(device, FALSE);
 
 	if (result != -ENOLINK)
 		start_autoscan(device);
 
-	connman_device_unref(device);
+	/*
+	 * If we are here then we were scanning; however, if we are
+	 * also mid-flight disabling the interface, then wifi_disable
+	 * has already cleared the device scanning state and
+	 * unreferenced the device, obviating the need to do it here.
+	 */
+
+	if (scanning == TRUE)
+		connman_device_unref(device);
 }
 
 static void scan_callback_hidden(int result,
