@@ -28,6 +28,7 @@
 #include <stdlib.h>
 
 #include <connman/ipconfig.h>
+#include <include/setting.h>
 
 #include <gdhcp/gdhcp.h>
 
@@ -82,7 +83,7 @@ static void dhcp_invalidate(struct connman_dhcp *dhcp, connman_bool_t callback)
 	if (dhcp == NULL)
 		return;
 
-	service = __connman_service_lookup_from_network(dhcp->network);
+	service = connman_service_lookup_from_network(dhcp->network);
 	if (service == NULL)
 		goto out;
 
@@ -197,7 +198,7 @@ static void lease_available_cb(GDHCPClient *dhcp_client, gpointer user_data)
 
 	DBG("Lease available");
 
-	service = __connman_service_lookup_from_network(dhcp->network);
+	service = connman_service_lookup_from_network(dhcp->network);
 	if (service == NULL) {
 		connman_error("Can not lookup service");
 		return;
@@ -226,7 +227,9 @@ static void lease_available_cb(GDHCPClient *dhcp_client, gpointer user_data)
 	if (option != NULL)
 		gateway = g_strdup(option->data);
 
-	prefixlen = __connman_ipconfig_netmask_prefix_len(netmask);
+	prefixlen = __connman_ipaddress_netmask_prefix_len(netmask);
+	if (prefixlen == 255)
+		connman_warn("netmask: %s is invalid", netmask);
 
 	DBG("c_address %s", c_address);
 
@@ -256,9 +259,12 @@ static void lease_available_cb(GDHCPClient *dhcp_client, gpointer user_data)
 	if (option != NULL)
 		domainname = g_strdup(option->data);
 
-	option = g_dhcp_client_get_option(dhcp_client, G_DHCP_HOST_NAME);
-	if (option != NULL)
-		hostname = g_strdup(option->data);
+	if (connman_setting_get_bool("AllowHostnameUpdates") == TRUE) {
+		option = g_dhcp_client_get_option(dhcp_client,
+						G_DHCP_HOST_NAME);
+		if (option != NULL)
+			hostname = g_strdup(option->data);
+	}
 
 	option = g_dhcp_client_get_option(dhcp_client, G_DHCP_NTP_SERVER);
 	ns_entries = g_list_length(option);
@@ -358,7 +364,7 @@ static void ipv4ll_available_cb(GDHCPClient *dhcp_client, gpointer user_data)
 
 	DBG("IPV4LL available");
 
-	service = __connman_service_lookup_from_network(dhcp->network);
+	service = connman_service_lookup_from_network(dhcp->network);
 	if (service == NULL)
 		return;
 
@@ -369,7 +375,7 @@ static void ipv4ll_available_cb(GDHCPClient *dhcp_client, gpointer user_data)
 	address = g_dhcp_client_get_address(dhcp_client);
 	netmask = g_dhcp_client_get_netmask(dhcp_client);
 
-	prefixlen = __connman_ipconfig_netmask_prefix_len(netmask);
+	prefixlen = __connman_ipaddress_netmask_prefix_len(netmask);
 
 	__connman_ipconfig_set_method(ipconfig, CONNMAN_IPCONFIG_METHOD_DHCP);
 	__connman_ipconfig_set_local(ipconfig, address);
@@ -440,9 +446,15 @@ static int dhcp_request(struct connman_dhcp *dhcp)
 
 	dhcp->dhcp_client = dhcp_client;
 
-	service = __connman_service_lookup_from_network(dhcp->network);
+	service = connman_service_lookup_from_network(dhcp->network);
 	ipconfig = __connman_service_get_ip4config(service);
 
+	/*
+	 * Clear the addresses at startup so that lease callback will
+	 * take the lease and set ip address properly.
+	 */
+	__connman_ipconfig_clear_address(ipconfig);
+
 	return g_dhcp_client_start(dhcp_client,
 				__connman_ipconfig_get_dhcp_address(ipconfig));
 }
