@@ -27,6 +27,8 @@
 
 #include <gdbus.h>
 
+#include <connman/agent.h>
+
 #include "connman.h"
 
 static connman_bool_t connman_state_idle;
@@ -159,7 +161,7 @@ static DBusMessage *remove_provider(DBusConnection *conn,
 	dbus_message_get_args(msg, NULL, DBUS_TYPE_OBJECT_PATH, &path,
 							DBUS_TYPE_INVALID);
 
-	err = __connman_provider_remove(path);
+	err = __connman_provider_remove_by_path(path);
 	if (err < 0)
 		return __connman_error_failed(msg, -err);
 
@@ -229,18 +231,12 @@ static DBusMessage *connect_provider(DBusConnection *conn,
 		connman_info("Session mode enabled: "
 				"direct provider connect disabled");
 
-		return __connman_error_failed(msg, -EINVAL);
+		return __connman_error_failed(msg, EINVAL);
 	}
 
 	err = __connman_provider_create_and_connect(msg);
-	if (err < 0) {
-		if (err == -EINPROGRESS) {
-			connman_error("Invalid return code from connect");
-			err = -EINVAL;
-		}
-
+	if (err < 0)
 		return __connman_error_failed(msg, -err);
-	}
 
 	return NULL;
 }
@@ -258,7 +254,7 @@ static DBusMessage *register_agent(DBusConnection *conn,
 	dbus_message_get_args(msg, NULL, DBUS_TYPE_OBJECT_PATH, &path,
 							DBUS_TYPE_INVALID);
 
-	err = __connman_agent_register(sender, path);
+	err = connman_agent_register(sender, path);
 	if (err < 0)
 		return __connman_error_failed(msg, -err);
 
@@ -278,7 +274,7 @@ static DBusMessage *unregister_agent(DBusConnection *conn,
 	dbus_message_get_args(msg, NULL, DBUS_TYPE_OBJECT_PATH, &path,
 							DBUS_TYPE_INVALID);
 
-	err = __connman_agent_unregister(sender, path);
+	err = connman_agent_unregister(sender, path);
 	if (err < 0)
 		return __connman_error_failed(msg, -err);
 
@@ -338,8 +334,12 @@ static DBusMessage *create_session(DBusConnection *conn,
 	DBG("conn %p", conn);
 
 	err = __connman_session_create(msg);
-	if (err < 0)
+	if (err < 0) {
+		if (err == -EINPROGRESS)
+			return NULL;
+
 		return __connman_error_failed(msg, -err);
+	}
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
@@ -403,13 +403,13 @@ static const GDBusMethodTable manager_methods[] = {
 	{ GDBUS_METHOD("GetTechnologies",
 			NULL, GDBUS_ARGS({ "technologies", "a(oa{sv})" }),
 			get_technologies) },
-	{ GDBUS_METHOD("RemoveProvider",
+	{ GDBUS_DEPRECATED_METHOD("RemoveProvider",
 			GDBUS_ARGS({ "provider", "o" }), NULL,
 			remove_provider) },
 	{ GDBUS_METHOD("GetServices",
 			NULL, GDBUS_ARGS({ "services", "a(oa{sv})" }),
 			get_services) },
-	{ GDBUS_ASYNC_METHOD("ConnectProvider",
+	{ GDBUS_DEPRECATED_ASYNC_METHOD("ConnectProvider",
 			      GDBUS_ARGS({ "provider", "a{sv}" }),
 			      GDBUS_ARGS({ "path", "o" }),
 			      connect_provider) },
@@ -426,7 +426,7 @@ static const GDBusMethodTable manager_methods[] = {
 	{ GDBUS_METHOD("UnregisterCounter",
 			GDBUS_ARGS({ "path", "o" }), NULL,
 			unregister_counter) },
-	{ GDBUS_METHOD("CreateSession",
+	{ GDBUS_ASYNC_METHOD("CreateSession",
 			GDBUS_ARGS({ "settings", "a{sv}" },
 						{ "notifier", "o" }),
 			GDBUS_ARGS({ "session", "o" }),
