@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2012  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -114,7 +114,7 @@ static gboolean check_plugin(struct connman_plugin_desc *desc,
 	return TRUE;
 }
 
-#include "builtin.h"
+#include <builtin.h>
 
 int __connman_plugin_init(const char *pattern, const char *exclude)
 {
