@@ -313,7 +313,7 @@ static void cleanup_user_config(struct user_config *user_config)
 }
 
 static int create_policy_config(struct connman_session *session,
-				connman_session_config_cb callback,
+				connman_session_config_func_t cb,
 				struct user_config *user_config)
 {
 	struct connman_session_config *config;
@@ -326,10 +326,10 @@ static int create_policy_config(struct connman_session *session,
 			return -ENOMEM;
 		}
 
-		return callback(session, config, user_config, 0);
+		return cb(session, config, user_config, 0);
 	}
 
-	return (*session->policy->create)(session, callback, user_config);
+	return (*session->policy->create)(session, cb, user_config);
 }
 
 static void probe_policy(struct connman_session_policy *policy)
@@ -359,6 +359,9 @@ static void remove_policy(struct connman_session_policy *policy)
 	gpointer key, value;
 	struct connman_session *session;
 
+	if (session_hash == NULL)
+		return;
+
 	DBG("policy %p name %s", policy, policy->name);
 
 	g_hash_table_iter_init(&iter, session_hash);
@@ -407,7 +410,7 @@ void connman_session_policy_unregister(struct connman_session_policy *policy)
 	remove_policy(policy);
 }
 
-int connman_session_set_default_config(struct connman_session_config *config)
+void connman_session_set_default_config(struct connman_session_config *config)
 {
 	config->priority = FALSE;
 	config->roaming_policy = CONNMAN_SESSION_ROAMING_POLICY_DEFAULT;
@@ -417,24 +420,14 @@ int connman_session_set_default_config(struct connman_session_config *config)
 	g_slist_free(config->allowed_bearers);
 	config->allowed_bearers = g_slist_prepend(NULL,
 				GINT_TO_POINTER(CONNMAN_SERVICE_TYPE_UNKNOWN));
-	if (config->allowed_bearers == NULL)
-		return -ENOMEM;
-
-	return 0;
 }
 
 struct connman_session_config *connman_session_create_default_config(void)
 {
 	struct connman_session_config *config;
 
-	config = g_try_new0(struct connman_session_config, 1);
-	if (config == NULL)
-		return NULL;
-
-	if (connman_session_set_default_config(config) < 0) {
-		g_free(config);
-		return NULL;
-	}
+	config = g_new0(struct connman_session_config, 1);
+	connman_session_set_default_config(config);
 
 	return config;
 }
@@ -514,7 +507,7 @@ static int filter_bearer(GSList *policy_bearers,
 	GSList *it;
 
 	if (policy_bearers == NULL)
-		goto clone;
+		return 0;
 
 	for (it = policy_bearers; it != NULL; it = it->next) {
 		policy = GPOINTER_TO_INT(it->data);
